#日本語UTF-8 Without BOM
module TAK2HATA
module SU2MITSUBA
module LIGHTINGS
class MITSUBA_PLACE_LIGHT
	def initialize( lighttype="spotlight" )
		@lighttype = lighttype
		@ip         = Sketchup::InputPoint.new
		@ip1        = Sketchup::InputPoint.new
		@ip2        = Sketchup::InputPoint.new
	end

	def activate
		@ip         = Sketchup::InputPoint.new
		view.invalidate if @drawn
		@drawn      = true
		self.reset(nil)
	end

	def resume(view)
	end

	def deactivate(view)
		view.invalidate if @drawn
	end

	def onCancel(flag, view)
		self.reset(view)
	end

	def reset(view)
		@state = 0
		@message = "Click Place Point"
		Sketchup.set_status_text( @message )
		@ip1.clear
		@ip2.clear
		@ip.clear
		if( view )
			view.tooltip = nil
			view.invalidate if @drawn
		end
		@drawn = false
	end
	def draw(view)
		if( @ip1.valid? )
			if( @ip1.display? )
				@ip1.draw(view)
				@drawn = true
			end
			if( @ip2.valid? )
				@ip2.draw(view) if( @ip2.display? )
				if view.inference_locked?
					line_width = 2
				else
					line_width = 1
				end
				view.line_width = line_width
				view.set_color_from_line(@ip1, @ip2)
				view.draw_line(@ip1.position, @ip2.position)
				@drawn = true
			end
		end
		Sketchup.set_status_text( @message )
	end
	def onMouseMove(flags, x, y, view)
		if( @state == 0 )
			@ip.pick view, x, y
			if( @ip != @ip1 )
				view.invalidate if( @ip.display? or @ip1.display? )
				@ip1.copy! @ip
				view.tooltip = @ip1.tooltip
			end
		else
			@ip2.pick view, x, y, @ip1
			view.tooltip = @ip2.tooltip if( @ip2.valid? )
			view.invalidate
		end
	end
	def onLButtonDown(flags, x, y, view)
		if( @state == 0 and @lighttype != "pointlight")
			@ip1.pick view, x, y
			if( @ip1.valid? )
				@state = 1
				@message = "Click Target Point"
			end
		else
			# create the line on the second click
			if @lighttype == "pointlight"
				place_light(@ip1.position, nil)
				self.reset(view)
			elsif( @ip2.valid? )
				place_light(@ip1.position, @ip2.position)
				self.reset(view)
			end
		end
		# Clear any inference lock
		view.lock_inference
	end
	def place_light( pt1 , pt2 )
		if @lighttype != "pointlight"
			return if not pt1.vector_to( pt2 ).valid?
		end
		model = Sketchup.active_model
		ents = model.active_entities
		if @lighttype == "pointlight"
			tr = Geom::Transformation.new(pt1)
		else
			zvec = pt1.vector_to( pt2 ).reverse
			tr = Geom::Transformation.new(pt1,zvec)
		end
		operation_name = "place mitsuba light"
		df = get_light_source_definition()
		return if not df
		Sketchup.active_model.start_operation operation_name
		Sketchup.active_model.active_entities.add_instance( df , tr )
		Sketchup.active_model.commit_operation
	end
	def get_light_source_definition( )
		dfs = Sketchup.active_model.definitions
		dirname = File.dirname(__FILE__)
		
		if @lighttype == "pointlight"
			compname = "T2H_PointLight"
		else
			compname = "T2H_SPOTLIGHT"
		end
		filename = File.join( dirname  , "component" , compname + ".skp" )
		df = dfs[compname]
		if not df
			df = dfs.load( filename )
		end
		return df
	end
end # Class MITSUBA_PLACE_LIGHT
class MITSUBA_EDIT_LIGHT

	def initialize(  )
		#p "initialize"

		#sels = Sketchup.active_model.selection
		#@lents = lents
		@ip = Sketchup::InputPoint.new
		show_light_dialog
	end

	def activate
		#p "activate"
		#@drawn      = true
		self.reset(nil)
	end

	def deactivate(view)
		#p "deactivate"
		@ltwdlg.close
		view.invalidate

	end

	def onCancel(flag, view)
		#p "onCancel"
		#view.invalidate
		Sketchup.active_model.selection.clear
		Sketchup.send_action("selectSelectionTool:")

	end

	def reset(view)
		@ip.clear
		Sketchup.set_status_text( "CLICK to SET SPOTLIGHT TARGET.( ESC to EXIT )" )
		@drawn = false
	end
	
	def onLButtonDown(flags, x, y, view)
		@ip.pick( view, x, y )
		
		@lents.find_all{|e2| e2.definition.name == "T2H_SPOTLIGHT"}.each{|e|
			etr = e.transformation
			position = etr.origin
			tvec = etr.zaxis
			target = @ip.position
			tvec2 = position.vector_to( target ) 
			otr = Geom::Transformation.new( position , tvec2.reverse )
			e.transformation = otr
		}
	end
	
	def draw(view)
		draw_spotlightpreview(view)
		#Sketchup.set_status_text( "EDITING LIGHTS" )
	end

	def draw_spotlightpreview(view)
		#view = Sketchup.active_model.active_view
		@lents.find_all{|e2| e2.definition.name == "T2H_SPOTLIGHT"}.each{|e|
			spotlight_infos = {}

			cutoffAngle = (@props["cutoffAngle"]||20).to_f##e.get_attribute("Mitsuba_light_setting","cutoffAngle",20).to_f
			beamWidth = (@props["beamWidth"]||15).to_f##e.get_attribute("Mitsuba_light_setting","beamWidth",15).to_f
			etr = e.transformation
			position = etr.origin
			tvec = etr.zaxis
			otr = Geom::Transformation.new( position , tvec.reverse )
			tvec.length = 2.m
			target = position - tvec
			dir_pts = []
			c_pts = []
			b_pts = []
			dir_pts.push [ position , target ]
			pt_zero = Geom::Point3d.new(0,0,0)
			vecz = Geom::Vector3d.new(0,0,-2.m)
			ctr = Geom::Transformation.rotation( position, etr.xaxis , cutoffAngle.degrees )
			btr = Geom::Transformation.rotation( position, etr.xaxis , beamWidth.degrees )
			c_stpt = target.transform( ctr )
			b_stpt = target.transform( btr )
			12.times{|i|
				rotr = Geom::Transformation.rotation( position , tvec , ( 30 * i ).degrees )
				cpt2 = c_stpt.transform( rotr )
				bpt2 = b_stpt.transform( rotr )
				dir_pts.push [ cpt2 , position ]
				dir_pts.push [ bpt2 ,  position ]
				c_pts.push cpt2
				b_pts.push bpt2
			}
			dir_pts.flatten!
			c_pts.push c_pts[0]
			b_pts.push b_pts[0]
			view.drawing_color = "Yellow"
			view.line_width = 1
			view.draw_lines( dir_pts )
			view.drawing_color = "blue"
			view.line_width = 2
			view.draw_polyline( c_pts )
			view.drawing_color = "cyan"
			view.line_width = 2
			view.draw_polyline( b_pts )
		}
		
		
	end
	def show_light_dialog()
		Sketchup.active_model.start_operation "Change Light Props"
		sels = Sketchup.active_model.selection
		@lents = sels.find_all{|e| e.kind_of? Sketchup::ComponentInstance and ( e.definition.name == "T2H_PointLight" or e.definition.name == "T2H_SPOTLIGHT" ) }
		
		view = Sketchup.active_model.active_view
		dflight_props  = default_light_props()

		#filesplit = (__FILE__).split(File::Separator)
		#basename = filesplit.pop
		#dirname = filesplit.join("/") + "/"
		
		basename = File.basename(__FILE__)
		dirname = File.dirname(__FILE__)
		htmfile = File.join( dirname, "su2objmtl_mitsuba_lightedit1.html" )

	    model = Sketchup.active_model
	    @ltwdlg = UI::WebDialog.new("Mitsuba Light setting", true, "MitsubaLightSetting", 355, 465, 150, 150, true)
	    if RUBY_PLATFORM[/darwin/]
			@ltwdlg.set_file('/'+htmfile)
		else
			@ltwdlg.set_file(htmfile)
		end
		@props = {}
	    @ltwdlg.add_action_callback("saveLightProp"){|d,p|
			get_props_from_webdialog(dflight_props)
			view.refresh
			#draw_spotlightpreview(view)
  	    }
		@ltwdlg.set_on_close{ 
			get_props_from_webdialog(dflight_props)
			saveLightProp(dflight_props)
			Sketchup.active_model.commit_operation
			Sketchup.active_model.selection.clear
			Sketchup.send_action("selectSelectionTool:")
		}
		get_props_from_selected( dflight_props )
	    @ltwdlg.show{
			itinitialize_props(dflight_props )
			view.refresh
			#draw_spotlightpreview(view)
		}
	end

	def itinitialize_props( dflight_props )
		dflight_props["spotlight"].each{|dfkey,dfarray|
			dftype = dfarray[0]
			dfval = dfarray[1]
			dflist = dfarray[2]
			val = @props[dfkey]||dfval
			if val == "*"
				@ltwdlg.execute_script("set_text('light_#{dfkey}','#{val.to_s.gsub(/\\/,'/')}')" )
			else
				if dftype == "rgb" or  dftype == "srgb"
					@ltwdlg.execute_script("set_text('light_#{dfkey}','#{val.to_s.gsub(/\\/,'/')}')" )
					@ltwdlg.execute_script("set_color('light_#{dfkey}','##{val.to_s.gsub(/\\/,'/')}')" )
				else
					@ltwdlg.execute_script("set_text('light_#{dfkey}','#{val.to_s.gsub(/\\/,'/')}')" )
				end
			end

		}
	end
	def get_props_from_webdialog( dflight_props )
		#props = {}
		dflight_props["spotlight"].keys.each{|dfkey|
			@props[dfkey] = @ltwdlg.get_element_value( "light_" + dfkey )
		}
		return# props
	end
	def check_light_type( e )
		if e.kind_of? Sketchup::ComponentInstance
			if "T2H_PointLight" == e.definition.name
				return "pointlight"
			elsif "T2H_SPOTLIGHT" == e.definition.name
				return "spotlight"
			end
		end
	end
	def get_props_from_selected( dflight_props)
		#props = {}
		@lents.each{|e|
			e_attdics =  e.attribute_dictionaries
			lighttype = check_light_type(e)
			if not e_attdics or not e_attdics[ "Mitsuba_light_setting" ]
				dflight_props[lighttype].each{|dfkey,dfarray|
					dftype = dfarray[0]
					dfval = dfarray[1]
					dflist = dfarray[2]
					e.set_attribute("Mitsuba_light_setting",dfkey,dfval)
					if @props[dfkey] and @props[dfkey] != dfval
						#プロパティが一致しない場合は*として扱う
						#これにより変更がない場合*が返却されるようにする
						@props[dfkey] = "*"
					else
						@props[dfkey] = dfval
					end
				}
			else
				dflight_props[lighttype].each{|dfkey,dfarray|
					dftype = dfarray[0]
					dfval = dfarray[1]
					dflist = dfarray[2]
					val = e.get_attribute("Mitsuba_light_setting",dfkey,dfval)
					if @props[dfkey] and @props[dfkey] != val
						#プロパティが一致しない場合は*として扱う
						#これにより変更がない場合*が返却されるようにする
						@props[dfkey] = "*"
					else
						@props[dfkey] = val
					end
				}
			end
		}
		return# props
	end
	def saveLightProp(dflight_props)
		@lents.each{|e|
			lighttype = check_light_type(e)
			dflight_props[lighttype].each{|dfkey,dfarray|
				dftype = dfarray[0]
				dfval = dfarray[1]
				dflist = dfarray[2]
				val = @props[dfkey]||dfval
				if val.to_s == "*"
					#この場合は上書きしない
				else
					e.set_attribute("Mitsuba_light_setting",dfkey,val)
				end
			}
		}
	end
	def default_light_props()
		dflightprops = {}
		dflightprops["pointlight"] = {
			"intensity_color"=>["rgb","FFFFFF","ColorPicker"],
			"intensity_scale"=>["float",1,"0.0|7.0"],
			"samplingWeight"=>["float",1,"2.0|64.0"]
		}
		dflightprops["spotlight"] = {
			"intensity_color"=>["rgb","FFFFFF","ColorPicker"],
			"intensity_scale"=>["float",1,"0.0|7.0"],
			"cutoffAngle"=>["float",20,"0.1|180.0"],
			"beamWidth"=>["float",15,"0.0|180.0"],
			"samplingWeight"=>["float",1,"2.0|64.0"]
		}
		return dflightprops
	end
end#class MITSUBA_EDIT_LIGHT

	def default_light_props()
		dflightprops = {}
		dflightprops["pointlight"] = {
			"intensity_color"=>["rgb","FFFFFF","ColorPicker"],
			"intensity_scale"=>["float",1,"0.0|7.0"],
			"samplingWeight"=>["float",1,"2.0|64.0"]
		}
		dflightprops["spotlight"] = {
			"intensity_color"=>["rgb","FFFFFF","ColorPicker"],
			"intensity_scale"=>["float",1,"0.0|7.0"],
			"cutoffAngle"=>["float",20,"0.1|180.0"],
			"beamWidth"=>["float",15,"0.0|180.0"],
			"samplingWeight"=>["float",1,"2.0|64.0"]
		}
		return dflightprops
	end
	def start_place_light_tool( lighttype="spotlight" )
		Sketchup.active_model.select_tool( MITSUBA_PLACE_LIGHT.new(lighttype) )
	end
	def start_edit_light_tool( )
		sels = Sketchup.active_model.selection
		lents = sels.find_all{|e| e.kind_of? Sketchup::ComponentInstance and ( e.definition.name == "T2H_PointLight" or e.definition.name == "T2H_SPOTLIGHT" ) }
		return if lents.size == 0
		Sketchup.active_model.select_tool( MITSUBA_EDIT_LIGHT.new() )
	end
	def place_force_vector()
		model = Sketchup.active_model
		dfs = model.definitions
		dirname = File.dirname(__FILE__)
		compname = "t2h_force_vector"
		filename = File.join( dirname  , "component" , compname + ".skp" )
		df = dfs[compname]
		if not df
			df = dfs.load( filename )
		end
		model.place_component df , true
	end
end#of module
end#module
end#module