module TAK2HATA
module SU2MITSUBA
module MATERIAL_EDIT
	class MITSUBA_MATERIAL_PICKER
		def initialize( wdlg )
			@wdlg = wdlg
			@ip = Sketchup::InputPoint.new
			cursor_path = File.join(File.dirname(__FILE__),"cursor_mat_picker.png")
			if File.exists?( cursor_path )
				@cursor_id = UI.create_cursor(cursor_path, 5, 17)
			end
		end
		def onSetCursor
			UI.set_cursor(@cursor_id)
		end
		def activate
			self.reset(nil)
		end

		def deactivate(view)
			@ip.clear
			view.invalidate
		end

		def onCancel(flag, view)
			Sketchup.active_model.selection.clear
			Sketchup.send_action("selectSelectionTool:")
		end

		def reset(view)
			@ip.clear
			Sketchup.set_status_text( "CLICK MATERIAL" )
			@drawn = false
		end
		
		def onLButtonDown(flags, x, y, view)
			camera = view.camera
			ph = view.pick_helper
			ph.do_pick(x, y)
			@ip.pick(view,x,y)
			#fc = ph.picked_face
			if ph.count != 0
				#entpath = ph.all_picked
				pt = @ip.position
				entpath = ph.path_at(0)
				mat = nil
				entpath.length.times{|i|
					e = entpath[entpath.length-i-1]
					if e.is_a?(Sketchup::Face)
						ang = e.normal.angle_between( pt.vector_to(camera.eye) )
						mat = e.material
						if ang > 90.degrees
							mat = e.back_material
						end
					else
						mat = e.material
					end
					break if mat != nil
				}
				if Sketchup.active_model.materials.current != mat
					@wdlg.execute_script("saveMaterial_all();")
					Sketchup.active_model.materials.current = mat
					@wdlg.execute_script("get_current_material();")
				end
			end
		end

	end
	def preview_image_refresh(cmat)
		previewimage =  get_preview_path(cmat)
		if File.exists?( previewimage )
			@mitsuba_material_wdlg.execute_script("update_preview('#{previewimage}');" )
			s = File.stat( previewimage )
			if s
				ptime = s.mtime.getlocal.strftime("%Y-%m-%d %H:%M:%S")
				@mitsuba_material_wdlg.execute_script("update_ptime('#{ptime}');" )
			end
		else
			previewimage = File.join(File.dirname(__FILE__),"matpreview","matpreview.png")
			@mitsuba_material_wdlg.execute_script("update_preview('#{previewimage}');" )
			@mitsuba_material_wdlg.execute_script("update_ptime('?');" )
		end
	end
	def get_render_path()
		filepath_arr = Sketchup.read_default("Mitsuba_Render_Exporter","RenderPath",[])
		return "" if filepath_arr == []
		begin
			render_path = filepath_arr.pack("H*")
		rescue
			render_path = ""
		end

		return render_path
	end
	def get_preview_path(cmat)
		
		#render_path = get_render_path()
		if @render_path and @render_path != ""
			if cmat
				matname = enc_kconv( cmat.name )
				previewimage =  File.join( @previewdir , matname + ".png" )
			else
				previewimage =  File.join( @previewdir , @default_mat_name + ".png" )
			end
		else
			previewimage = ""
		end
		return previewimage
	end
	def preview_start(cmat)
		@texnames_bymat={}
		@textureslist={}

		get_render_path_ini()
		return if @render_path == nil or @render_path == ""
		folder_setup()
		
		#filesplit = (__FILE__).split(File::Separator)
		basename = File.basename(__FILE__)
		dirname = File.dirname(__FILE__)
		render_dir = File.dirname( @render_path )
		if RUBY_PLATFORM[/darwin/]
			render_path2 = File.join(@render_path,"Contents","MacOS","mitsuba")
		else
			render_path2 = File.join( render_dir , "mitsuba.exe" )
		end
		dirname = File.join( dirname , "matpreview" )
		#prv_dir = @previewdir
		if cmat
			matname = enc_kconv( cmat.name )
		else
			matname = @default_mat_name
		end
		previewbase = File.join( dirname , "matpreview_new.xml")
		dirname = "#{dirname}\\"
		previewfile = File.join( @previewdir , "matpreview2.xml")
		previewimage =  get_preview_path(cmat)

		#@texture_exportpath = File.join( render_dir , "su_render" ,"tx_default")

		dfprops , dfcoatprops ,	dflightprops , dfdipoleprops , dfotherprops = mitsuba_mat_properties_list()

		f = open(previewbase,"r")
		tbuff = f.read
		f.close
		
		matcode , matcode2 , trans= export_this_material( cmat , "preview_material" , dfprops , dfcoatprops,	dflightprops , dfdipoleprops ,dfotherprops, 0.5 )
		export_textures( true )

		texcode = ""
		@textureslist.each{|key,val|
			texcode = texcode + val
		}

		texcode.gsub!(/\\/,"\\\\")
		matcode.gsub!(/\\/,"\\\\")
		matcode2.gsub!(/\\/,"\\\\")
		#matcode3 = matcode + matcode2
		tbuff = tbuff.split(/\<\!-- preview_texture --\>/).join(texcode)
		tbuff = tbuff.split(/\<\!-- preview_material --\>/).join(matcode)
		tbuff = tbuff.split(/\<\!-- preview_sss --\>/).join(matcode2)
		tbuff = tbuff.split(/\<\!-- pluginpath --\>/).join(dirname)
		tbuff = tbuff.split(/\<\!-- preview_material_name --\>/).join(matname)
		
		f = open(previewfile,"w+")
		f.write(tbuff)
		f.close
		if RUBY_PLATFORM[/darwin/]
		   commandline = %Q{#{render_path2} -q -r2 -b16 -o '#{previewimage.gsub(/\\/,'/')}' '#{previewfile.gsub(/\\/,'/')}'}
		else
			commandline = "\"#{render_path2.gsub(/\//,"\\")}\" -q -r2 -b16 -o \"#{previewimage.gsub(/\//,"\\")}\" \"#{previewfile.gsub(/\//,"\\")}\" "
		end
		system(commandline)

		preview_image_refresh(cmat)
	end

############################################################################################

	def get_prop_from_wdlg(cmat)
		selected_type = @mitsuba_material_wdlg.get_element_value("mattype_list")
		selected_coat = @mitsuba_material_wdlg.get_element_value("coat_list")
		selected_light = @mitsuba_material_wdlg.get_element_value("light_list")
		selected_dipole = @mitsuba_material_wdlg.get_element_value("dipole_list")
		dfprops , dfcoatprops ,	dflightprops , dfdipoleprops , dfotherprops = mitsuba_mat_properties_list()
		props = {}
		coatprops = {}
		lightprops = {}
		dipoleprops = {}
		dfprops[selected_type].keys.each{|dfkey|
			vtype = dfprops[selected_type][dfkey][0]
			props[dfkey] = @mitsuba_material_wdlg.get_element_value( selected_type + "_" + dfkey )
			if vtype == "float" or vtype == "integer"
				props[dfkey].gsub!(/\,/,".")
			end
		}
		if cmat == nil
			props["default_mat_color"] = @mitsuba_material_wdlg.get_element_value( "default_material_color" )
		end
		if selected_coat != "none"
			dfcoatprops[selected_coat].keys.each{|dfkey|
				vtype = dfcoatprops[selected_coat][dfkey][0]
				coatprops[dfkey] = @mitsuba_material_wdlg.get_element_value( selected_coat + "_" + dfkey )
				if vtype == "float" or vtype == "integer"
					coatprops[dfkey].gsub!(/\,/,".")
				end
			}
		end
		if selected_light != "none"
			dflightprops[selected_light].keys.each{|dfkey|
				vtype = dflightprops[selected_light][dfkey][0]
				lightprops[dfkey] = @mitsuba_material_wdlg.get_element_value( selected_light + "_" + dfkey )
				if vtype == "float" or vtype == "integer"
					lightprops[dfkey].gsub!(/\,/,".")
				end
			}
		end
		if selected_dipole != "none"
			dfdipoleprops[selected_dipole].keys.each{|dfkey|
				vtype = dfdipoleprops[selected_dipole][dfkey][0]
				dipoleprops[dfkey] = @mitsuba_material_wdlg.get_element_value( selected_dipole + "_" + dfkey )
				if vtype == "float" or vtype == "integer"
					dipoleprops[dfkey].gsub!(/\,/,".")
				end
			}
		end
		dfotherprops["other"].keys.each{|dfkey|
			vtype = dfotherprops["other"][dfkey][0]
			props[dfkey] = @mitsuba_material_wdlg.get_element_value( "other_" + dfkey )
			if dfkey == "bump_width" or dfkey == "bump_height"
			elsif vtype == "float" or vtype == "integer"
				props[dfkey].gsub!(/\,/,".")
			end
		}
		#props["bump"] = @mitsuba_material_wdlg.get_element_value( "bump" )
		#props["bumpscale"] = @mitsuba_material_wdlg.get_element_value( "bump_scale" )
		#props["bumpwidth"] = @mitsuba_material_wdlg.get_element_value( "bump_width" )
		#props["bumpheight"] = @mitsuba_material_wdlg.get_element_value( "bump_height" )
		#props["mask"] = @mitsuba_material_wdlg.get_element_value( "other_mask" )
		#props["twosided"] = @mitsuba_material_wdlg.get_element_value( "other_twosided" )

		props["bsdf_type"] = selected_type
		coatprops["coat_type"] = selected_coat
		lightprops["light_type"] = selected_light
		dipoleprops["dipole_type"] = selected_dipole
		#p props
		return props,coatprops,lightprops,dipoleprops
	end
	def material_edit_show_webdialog(coat_type="",bsdf_type="",light_type="",dipole_type="",starttab="0")
		@default_mat_name = "su2objmtl_default_material"
		if @mitsuba_material_wdlg
			@mitsuba_material_wdlg.bring_to_front
			return
		end
		model = Sketchup.active_model
		@mitsuba_material_wdlg = UI::WebDialog.new("Mitsuba Material Editor", true, "Mitsuba Material Editor", 430, 770, 0, 0, true)
		mats = model.materials
		cmat = mats.current
		#if not cmat
		#	UI.messagebox("Unable to edit the Default material")
		#	return 
		#end
		get_render_path_ini()
		folder_setup()

		htmfile = File.join(File.dirname(__FILE__),'su2objmtl_mitsuba_materialedit2.html')
		#@mitsuba_material_wdlg.set_html(tbuff)
		if RUBY_PLATFORM[/darwin/]
			@mitsuba_material_wdlg.set_file('/'+htmfile)
		else
			@mitsuba_material_wdlg.set_file(htmfile)
		end
		
		@mitsuba_material_wdlg.set_on_close{
			props ,coatprops,lightprops,dipoleprops = get_prop_from_wdlg(cmat)
			save_mat_props( cmat ,props ,coatprops,lightprops,dipoleprops )
			@mitsuba_material_wdlg = nil
		}
	    @mitsuba_material_wdlg.add_action_callback("get_current_material"){|d,p|
			##@mitsuba_material_wdlg.close
			##material_edit_show_webdialog()
			cmat = model.materials.current
			material_list_update()
			preview_image_refresh(cmat)
		}
		@mitsuba_material_wdlg.add_action_callback("pick_material"){|d,p|
			matpicker=MITSUBA_MATERIAL_PICKER.new(@mitsuba_material_wdlg)
			Sketchup.active_model.select_tool(matpicker)
		}
	    @mitsuba_material_wdlg.add_action_callback("clear_all"){|d,p|
			if UI.messagebox("Clear All Properties on this material?",MB_OKCANCEL) == 1
				clear_all_mat_props(cmat)
				material_list_update()
				preview_image_refresh(cmat)
			end
		}
		@mitsuba_material_wdlg.add_action_callback("change_mat_list"){|d,p|
			selected_mat = @mitsuba_material_wdlg.get_element_value("material_list")
			if selected_mat == ""
				model.materials.current = nil
			else
				model.materials.current = selected_mat
			end
			cmat = model.materials.current
			material_list_update()
			preview_image_refresh(cmat)
	    }

	    @mitsuba_material_wdlg.add_action_callback("save_mat_param"){|d,p|
			props ,coatprops,lightprops,dipoleprops = get_prop_from_wdlg(cmat)
			save_mat_props( cmat ,props ,coatprops,lightprops,dipoleprops )
  	    }
	    @mitsuba_material_wdlg.add_action_callback("save_mat_file"){|d,p|
			props ,coatprops,lightprops,dipoleprops = get_prop_from_wdlg(cmat)
			save_mat_file( cmat ,props ,coatprops,lightprops,dipoleprops )
  	    }
	    @mitsuba_material_wdlg.add_action_callback("load_mat_file"){|d,p|
			load_mat_file(cmat)
			material_list_update()
			preview_image_refresh(cmat)
  	    }
	    @mitsuba_material_wdlg.add_action_callback("selectfile"){|d,p|
			open_filename = UI.openpanel"Select Mapfile", "", ""
			return if open_filename == nil
			open_filename.gsub!(/\\/,'/')
			cmd = "document.getElementById('#{p}').value='#{open_filename}';update_cbox();"
			@mitsuba_material_wdlg.execute_script( cmd )
  	    }
	    @mitsuba_material_wdlg.add_action_callback("selectfile2"){|d,p|
			open_filename = UI.openpanel"Select Mapfile", "", ""
			if open_filename and open_filename != ""
				open_filename.gsub!(/\\/,'/')
				cmd="set_text('#{p}','#{open_filename}');update_cbox();"
				@mitsuba_material_wdlg.execute_script( cmd )
			end
  	    }
		#@mitsuba_material_wdlg.add_action_callback("fit_bump_size"){|d,p|
		#	if cmat.texture
		#		obw = length_to_decim(cmat.texture.width.to_l)
		#		obh = length_to_decim(cmat.texture.height.to_l)
		#		@mitsuba_material_wdlg.execute_script( %Q{set_text("other_bump_width","#{obw.to_s.gsub(/[^\d.,-]/,"")}")} )
		#		@mitsuba_material_wdlg.execute_script( %Q{set_text("other_bump_height","#{obh.to_s.gsub(/[^\d.,-]/,"")}")} )
		#	end
		#}
		@mitsuba_material_wdlg.add_action_callback("fit_image_size"){|d,p|
			if cmat.texture
				obw = length_to_decim(cmat.texture.width.to_l)
				obh = length_to_decim(cmat.texture.height.to_l)
				@mitsuba_material_wdlg.execute_script( %Q{set_text("image_width","#{obw.to_s.gsub(/[^\d.,-]/,"")}")} )
				@mitsuba_material_wdlg.execute_script( %Q{set_text("image_height","#{obh.to_s.gsub(/[^\d.,-]/,"")}")} )
			end
		}
		@mitsuba_material_wdlg.add_action_callback("material_preview"){|d,p| preview_start(cmat) }
	    @mitsuba_material_wdlg.show{ 
			material_list_update()
			@mitsuba_material_wdlg.execute_script( '$( "#tabs" ).tabs({ active: ' + starttab + ' });' )
			preview_image_refresh(cmat)
		}
	end
	def clear_all_mat_props(cmat)
		if cmat
			mat_attdics = cmat.attribute_dictionaries
			if mat_attdics
				mat_attdics.delete("Mitsuba_material_setting")
				mat_attdics.delete("Mitsuba_coat_setting")
				mat_attdics.delete("Mitsuba_light_setting")
				mat_attdics.delete("Mitsuba_dipole_setting")
			end
		else
			mat_attdics = Sketchup.active_model.attribute_dictionaries
			mat_attdics.delete("Mitsuba_default_material_setting")
			mat_attdics.delete("Mitsuba_default_material_coat_setting")
			mat_attdics.delete("Mitsuba_default_material_light_setting")
			mat_attdics.delete("Mitsuba_default_material_dipole_setting")
		end
	end
	def set_prop_to_webdialog(typename,propname,propval,proplist)
		type = proplist[0]
		val = propval||proplist[1]
		#val.gsub!(/\\/,'/')
		
		dfval = proplist[2]
		if dfval != "Origin"
			if type == "float" or type == "integer" or type == "blackbody"
				@mitsuba_material_wdlg.execute_script( "set_text('#{typename}_#{propname}','#{val}')")
			elsif type == "string" or type == "boolean"
				@mitsuba_material_wdlg.execute_script( "selectListbox('#{typename}_#{propname}','#{val}')")
			elsif type == "srgb" or type == "rgb"
				@mitsuba_material_wdlg.execute_script( "set_text('#{typename}_#{propname}','#{val}')")
				#@mitsuba_material_wdlg.execute_script( "set_color('#{typename}_#{propname}','##{val}')")
			elsif type == "file"
				@mitsuba_material_wdlg.execute_script( "set_text('#{typename}_#{propname}','#{val.gsub(/\\/,'/')}')")
			end
		end
	end
	def jsescape( str1 )
		str = "" + str1
		str.gsub!(/'/,"\\\\'")
		return str
	end
	def material_list_update()
		mats = Sketchup.active_model.materials
		dfprops , dfcoatprops ,dflightprops , dfdipoleprops , dfotherprops= mitsuba_mat_properties_list()
		@mitsuba_material_wdlg.execute_script( "document.getElementById('material_list').length =0;" )
		@mitsuba_material_wdlg.execute_script( "document.getElementById('material_list').options[0] = new Option('','')" )
		cmat = mats.current
		chk = false
		mats.each{|mat|
			if cmat != nil and cmat == mat
				chk = true
			end
			@mitsuba_material_wdlg.execute_script( "document.getElementById('material_list').options[document.getElementById('material_list').length] = new Option('#{jsescape(mat.name)}','#{jsescape(mat.name)}')" )
		}
		if chk == false and cmat != nil
			@mitsuba_material_wdlg.execute_script( "document.getElementById('material_list').options[document.getElementById('material_list').length] = new Option('#{jsescape(cmat.name)}','#{jsescape(cmat.name)}')" )
		end
		props ,coatprops,lightprops,dipoleprops = get_prop_from_material(cmat)
		bsdf = props["bsdf_type"]||"diffuse"
		coat = coatprops["coat_type"]||"none"
		light = lightprops["light_type"]||"none"
		dipole = dipoleprops["dipole_type"]||"none"

		if cmat
			@mitsuba_material_wdlg.execute_script("selectListbox('material_list','#{jsescape(cmat.name)}')" )
		else
			@mitsuba_material_wdlg.execute_script("selectListbox('material_list','')" )
		end
		
		@mitsuba_material_wdlg.execute_script("selectListbox('mattype_list','#{bsdf}')" )
		@mitsuba_material_wdlg.execute_script("changedMatType('#{bsdf}')" )

		dfotherprops["other"].each{|propname,proplist| set_prop_to_webdialog("other",propname,props[propname],proplist) }
		dfprops[bsdf].each{|propname,proplist| set_prop_to_webdialog(bsdf,propname,props[propname],proplist) }
		if props["default_mat_color"]
			set_prop_to_webdialog("default_material","color",props["default_mat_color"],["srgb"])
		end
		if coat != ""
			@mitsuba_material_wdlg.execute_script("selectListbox('coat_list','#{coat}')" )
			@mitsuba_material_wdlg.execute_script("changedCoat('#{coat}')" )
			dfcoatprops[coat].each{|propname,proplist| set_prop_to_webdialog(coat,propname,coatprops[propname],proplist) }
		end
		if light != ""
			@mitsuba_material_wdlg.execute_script("selectListbox('light_list','#{light}')" )
			@mitsuba_material_wdlg.execute_script("changedLight('#{light}')" )
			dflightprops[light].each{|propname,proplist| set_prop_to_webdialog(light,propname,lightprops[propname],proplist) }
		end
		if dipole != ""
			@mitsuba_material_wdlg.execute_script("selectListbox('dipole_list','#{dipole}')" )
			@mitsuba_material_wdlg.execute_script("changedDipole('#{dipole}')" )
			dfdipoleprops[dipole].each{|propname,proplist| set_prop_to_webdialog(dipole,propname,dipoleprops[propname],proplist) }
		end

	end
	def load_mat_file(cmat)
		path_to_load_to = UI.openpanel( "Load Material File", "", "*.mtbmat.txt" )
		return if not path_to_load_to
		matfile = File.new(path_to_load_to, "r")
		if not File.exists?(matfile)
			UI.messagebox("File could not create ,it was caused by the file path")
			return
		end
		list = ["##MAIN_PROPS##","##COAT_PROPS##","##LIGHT_PROPS##","##DIPOLE_PROPS##","##END##"]
		chk = -1
		props = {}
		coatprops = {}
		dipoleprops = {}
		lightprops = {}
		while line = matfile.gets
			line.strip!
			line.chomp!
			if list.include?(line)
				chk = list.index(line)
				next
			end
			line2 = matfile.gets
			line2.strip!
			line2.chomp!
			if chk == 0
				props[line] = line2
			elsif chk == 1
				coatprops[line] = line2
			elsif chk == 2
				lightprops[line] = line2
			elsif chk == 3
				dipoleprops[line] = line2
			end
		end
		matfile.close
		#save_mat_props( cmat ,props ,coatprops,lightprops,dipoleprops )
	end
	def save_mat_file( cmat ,props,coatprops,lightprops,dipoleprops )
		if cmat
			filename = enc_kconv(cmat.name) + ".mtbmat.txt"
		else
			filename = @default_mat_name + ".mtbmat.txt"
		end
		path_to_save_to = UI.savepanel( "Save Material File", "", filename )
		return if not path_to_save_to
		matfile = File.new(path_to_save_to, "w")
		if not File.exists?(matfile)
			UI.messagebox("File could not create ,it was caused by the file path")
			return
		end
		matfile.puts %Q{##MAIN_PROPS##}
		props.each{|key,val|
			matfile.puts key
			matfile.puts val
		}
		matfile.puts %Q{##COAT_PROPS##}
		coatprops.each{|key,val|
			matfile.puts key
			matfile.puts val
		}
		matfile.puts %Q{##LIGHT_PROPS##}
		lightprops.each{|key,val|
			matfile.puts key
			matfile.puts val
		}
		matfile.puts %Q{##DIPOLE_PROPS##}
		dipoleprops.each{|key,val|
			matfile.puts key
			matfile.puts val
		}
		matfile.puts %Q{##END##}
		matfile.close
	end
	def save_mat_props( cmat ,props,coatprops,lightprops,dipoleprops )
		cmatname = cmat.name if cmat
		Sketchup.active_model.start_operation("mitsuba material edit:#{cmatname||'default material'}")
		if cmat
			mat_attdics =  cmat.attribute_dictionaries
			if mat_attdics
				mat_attdics.delete("Mitsuba_material_setting")
				mat_attdics.delete("Mitsuba_coat_setting")
				mat_attdics.delete("Mitsuba_light_setting")
				mat_attdics.delete("Mitsuba_dipole_setting")
			end
			props.each{|key,val|
				cmat.set_attribute("Mitsuba_material_setting",key,val)
			}
			coatprops.each{|key,val|
				cmat.set_attribute("Mitsuba_coat_setting",key,val)
			}
			lightprops.each{|key,val|
				cmat.set_attribute("Mitsuba_light_setting",key,val)
			}
			dipoleprops.each{|key,val|
				cmat.set_attribute("Mitsuba_dipole_setting",key,val)
			}
		else
			model = Sketchup.active_model
			mat_attdics = model.attribute_dictionaries
			
			if mat_attdics
				mat_attdics.delete("Mitsuba_default_material_setting")
				mat_attdics.delete("Mitsuba_default_material_coat_setting")
				mat_attdics.delete("Mitsuba_default_material_light_setting")
				mat_attdics.delete("Mitsuba_default_material_dipole_setting")
			end
			props.each{|key,val|
				model.set_attribute("Mitsuba_default_material_setting",key,val)
			}
			coatprops.each{|key,val|
				model.set_attribute("Mitsuba_default_material_coat_setting",key,val)
			}
			lightprops.each{|key,val|
				model.set_attribute("Mitsuba_default_material_light_setting",key,val)
			}
			dipoleprops.each{|key,val|
				model.set_attribute("Mitsuba_default_material_dipole_setting",key,val)
			}
		end

		#mat_attdics[ "Mitsuba_material_setting" ] = props if props
		#mat_attdics[ "Mitsuba_coat_setting" ] = coatprops if coatprops
		Sketchup.active_model.commit_operation
	end
	def get_prop_from_material(mat)
		props = {}
		coatprops = {}
		dipoleprops = {}
		lightprops = {}
		if not mat
			mat_attdics = Sketchup.active_model.attribute_dictionaries
			if mat_attdics
				if mat_attdics[ "Mitsuba_default_material_setting" ]
					mat_attdics[ "Mitsuba_default_material_setting" ].each{|key,val|
						props[key] = val
					}
				end
				if mat_attdics[ "Mitsuba_default_material_coat_setting" ]
					mat_attdics[ "Mitsuba_default_material_coat_setting" ].each{|key,val| coatprops[key] = val }
				end
				if mat_attdics[ "Mitsuba_default_material_light_setting" ]
					mat_attdics[ "Mitsuba_default_material_light_setting" ].each{|key,val| lightprops[key] = val }
				end
				if mat_attdics[ "Mitsuba_default_material_dipole_setting" ]
					mat_attdics[ "Mitsuba_default_material_dipole_setting" ].each{|key,val| dipoleprops[key] = val }
				end
			end
		else
			mat_attdics =  mat.attribute_dictionaries
			if mat_attdics
				if mat_attdics[ "Mitsuba_material_setting" ]
					mat_attdics[ "Mitsuba_material_setting" ].each{|key,val| props[key] = val }
				end
				if mat_attdics[ "Mitsuba_coat_setting" ]
					mat_attdics[ "Mitsuba_coat_setting" ].each{|key,val| coatprops[key] = val }
				end
				if mat_attdics[ "Mitsuba_light_setting" ]
					mat_attdics[ "Mitsuba_light_setting" ].each{|key,val| lightprops[key] = val }
				end
				if mat_attdics[ "Mitsuba_dipole_setting" ]
					mat_attdics[ "Mitsuba_dipole_setting" ].each{|key,val| dipoleprops[key] = val }
				end
			end
		end
		return props ,coatprops,lightprops,dipoleprops
	end
	def mitsuba_mat_properties_list()
#		dfprops , dfcoatprops ,	dflightprops , dfdipoleprops = mitsuba_mat_properties_list()
		#mask,,"opacity"=>0.5
		#twosided,,
		#dfprops["coating"] = {"intIOR"=>1.5046,"extIOR"=>1.000277,"thickness"=>1.0,"sigma"=>0,"specularTransmittance"=>1.0}
		#dfprops["roughcoating"] = {"distribution"=>"beckmann","alpha"=0.1,"intIOR"=>1.5046,"extIOR"=>1.000277,"thickness"=>1.0,"sigma"=>0,"specularTransmittance"=>1.0}
		#dfprops["bump"] = {"texture"=>nil}

		dfprops = {}
		dfcoatprops = {}
		dflightprops = {}
		dfdipoleprops = {}
		dfotherprops = {}
		dfdipoleprops["none"] = {}
		dfdipoleprops["dipole"] = {
			"sigmaS"		=>["rgb","FFFFFF","ColorPicker"],
			"sigmaA"		=>["rgb","0D0D0D","ColorPicker"],
			"intIOR"		=>["float",1.5046,"1.0|3.0"],
			"extIOR"		=>["float",1.000277,"1.0|3.0"],
			"scale"		=>["float",10,"0.0|100.0"],
			"irrSamples"=>["integer",16,"16|256|16"]
		}

		dflightprops["none"] = {}
		dflightprops["arealight_blackbody"] = {
			"temperature"		=>["blackbody",6000,"1000|7000"],
			"scale"		=>["integer",-3,"-8|8"],
			"samplingWeight"=>["float",1.0,"0.1|4.0"]
		}
		dflightprops["arealight_srgb"] = {
			"radiance"		=>["srgb","FFFFFF","ColorPicker"],
			"scale2"		=>["float",1.0,"0|5"],
			"samplingWeight"=>["float",1.0,"0.1|4.0"]
		}

		dfprops["diffuse"] = {"reflectance"=>["srgb","808080","Origin"]}
		dfprops["difftrans"] = {"transmittance"=>["srgb","808080","Origin"]}
		dfprops["hk"] = {
			"sigmaS"		=>["rgb","FFFFFF","Origin"],
			"sigmaA"		=>["rgb","0D0D0D","ColorPicker"],
			"thickness"		=>["float",1.0,"0.0|3.0"]
		}
		dfprops["roughdiffuse"] = {
			"reflectance"	=>["srgb","808080","Origin"],
			"alpha"			=>["float",0.2,"0.0|1.0"],
			"useFastApprox"=>["boolean",false,"true|false"]
		}
		dfprops["dielectric"] = {
			"intIOR"			=>["float",1.5046,"1.0|3.0"],
			"extIOR"			=>["float",1.000277,"1.0|3.0"],
			"specularReflectance"=>["srgb","FFFFFF","ColorPicker"],
			"specularTransmittance"=>["srgb","FFFFFF","Origin"]
		}
		dfprops["thindielectric"] = {
			"intIOR"			=>["float",1.5046,"1.0|3.0"],
			"extIOR"			=>["float",1.000277,"1.0|3.0"],
			"specularReflectance" => ["srgb","FFFFFF","ColorPicker"],
			"specularTransmittance" => ["srgb","FFFFFF","Origin"]
		}
		dfprops["roughdielectric"] = {
			"distribution"	=>["string","as","as"],#beckmann|ggx|phong|
			"alpha"			=>["float",0.1,"0.0|1.0"],
			"alphaU"		=>["float",0.1,"0.0|1.0"],
			"alphaV"		=>["float",0.1,"0.0|1.0"],
			"intIOR"		=>["float",1.5046,"1.0|3.0"],
			"extIOR"		=>["float",1.000277,"1.0|3.0"],
			"specularReflectance"=>["srgb","FFFFFF","ColorPicker"],
			"specularTransmittance"=>["srgb","FFFFFF","Origin"]
		}
		dfprops["conductor"] = {
			"material"			=>["string","none","none|a-C|Ag|Al|AlAs|AlSb|Au|Be|Cr|CsI|Cu|Cu20|CuO|d-C|Hg|HgTe|Ir|K|Li|MgO|Mo|Na_palik|Nb|Ni_palik|Rh|Se|SiC|SnTe|Ta|Te|ThF4|TiC|TiN|TiO2|VC|V_palik|VN|W"],
			"specularReflectance"=>["srgb","FFFFFF","Origin"]
		}

		dfprops["roughconductor"] = {
			"distribution"		=>["string","as","as"],#beckmann|ggx|phong|
			"alpha"				=>["float",0.1,"0.0|1.0"],
			"alphaU"			=>["float",0.1,"0.0|1.0"],
			"alphaV"			=>["float",0.1,"0.0|1.0"],
			"material"			=>["string","none","none|a-C|Ag|Al|AlAs|AlSb|Au|Be|Cr|CsI|Cu|Cu20|CuO|d-C|Hg|HgTe|Ir|K|Li|MgO|Mo|Na_palik|Nb|Ni_palik|Rh|Se|SiC|SnTe|Ta|Te|ThF4|TiC|TiN|TiO2|VC|V_palik|VN|W"],
			"specularReflectance"=>["srgb","FFFFFF","Origin"]
		}

		dfprops["plastic"] = {
			"intIOR"			=>["float",1.49,"1.0|3.0"],
			"extIOR"			=>["float",1.000277,"1.0|3.0"],
			"specularReflectance"=>["srgb","FFFFFF","ColorPicker"],
			"diffuseReflectance"=>["srgb","808080","Origin"],
			"nonlineear"		=>["boolean",false,"true|false"]
		}
		dfprops["roughplastic"] = {
			"distribution"		=>["string","beckmann","beckmann|ggx|phong"],
			"alpha"				=>["float",0.1,"0.0|1.0"],
			"intIOR"			=>["float",1.49,"1.0|3.0"],
			"extIOR"			=>["float",1.000277,"1.0|3.0"],
			"specularReflectance"=>["srgb","FFFFFF","ColorPicker"],
			"diffuseReflectance"=>["srgb","808080","Origin"],
			"nonlineear"		=>["boolean",false,"true|false"]
		}
		dfprops["phong"] = {
			"exponent"			=>["float",30,"1|100"],
			"specularReflectance"=>["srgb","333333","ColorPicker"],
			"diffuseReflectance"=>["srgb","808080","Origin"]
		}
		dfprops["ward"] = {
			"variant"		=>["string","balanced","ward|ward-duer|balanced"],
			"alphaU"		=>["float",0.1,"0.0|1.0"],
			"alphaV"		=>["float",0.1,"0.0|1.0"],
			"specularReflectance"	=>["srgb","333333","ColorPicker"],
			"diffuseReflectance"	=>["srgb","808080","Origin"]
		}
		dfprops["irawan-charmeuse"] = {
			"repeatU"			=>["float",1,"1.0|100.0|0.1"],
			"repeatV"			=>["float",1,"1.0|100.0|0.1"],
			"warp_kd"	=>["srgb","0A0A0A","Origin"],
			"warp_ks"	=>["srgb","000000","ColorPicker"],
			"weft_kd"	=>["srgb","000000","ColorPicker"],
			"weft_ks"	=>["srgb","808080","ColorPicker"]
		}
		dfprops["irawan-denim"] = {
			"repeatU"			=>["float",1,"1.0|100.0|0.1"],
			"repeatV"			=>["float",1,"1.0|100.0|0.1"],
			"warp_kd"	=>["srgb","0A0A0A","Origin"],
			"warp_ks"	=>["srgb","000000","ColorPicker"],
			"weft_kd"	=>["srgb","000000","ColorPicker"],
			"weft_ks"	=>["srgb","808080","ColorPicker"]
		}
		dfprops["irawan-shantung"] = {
			"repeatU"			=>["float",1,"1.0|100.0|0.1"],
			"repeatV"			=>["float",1,"1.0|100.0|0.1"],
			"warp_kd"	=>["srgb","0A0A0A","Origin"],
			"warp_ks"	=>["srgb","000000","ColorPicker"],
			"weft_kd"	=>["srgb","000000","ColorPicker"],
			"weft_ks"	=>["srgb","808080","ColorPicker"]
		}
		dfprops["irawan-gabardine"] = {
			"repeatU"			=>["float",1,"1.0|100.0|0.1"],
			"repeatV"			=>["float",1,"1.0|100.0|0.1"],
			"kd"	=>["srgb","808080","Origin"],
			"ks"	=>["srgb","000000","ColorPicker"]
		}
		dfprops["irawan-polyester"] = {
			"repeatU"			=>["float",1,"1.0|100.0|0.1"],
			"repeatV"			=>["float",1,"1.0|100.0|0.1"],
			"kd"	=>["srgb","808080","Origin"],
			"ks"	=>["srgb","000000","ColorPicker"]
		}
		dfprops["irawan-twill"] = {
			"repeatU"			=>["float",1,"1.0|100.0|0.1"],
			"repeatV"			=>["float",1,"1.0|100.0|0.1"],
			"kd"	=>["srgb","808080","Origin"],
			"ks"	=>["srgb","000000","ColorPicker"]
		}
		dfcoatprops["coating"] = {
			"intIOR"		=>["float",1.5046,"1.0|3.0"],
			"extIOR"		=>["float",1.000277,"1.0|3.0"],
			"thickness"		=>["float",1.0,"0.1|3.0"],
			"sigmaA"		=>["srgb","000000","ColorPicker"],
			"specularTransmittance"=>["srgb","FFFFFF","ColorPicker"]
		}
		dfcoatprops["roughcoating"] = {
			"distribution"	=>["string","beckmann","beckmann|ggx|phong"],
			"alpha"			=>["float",0.1,"0.0|1.0"],
			"intIOR"		=>["float",1.5046,"1.0|3.0"],
			"extIOR"		=>["float",1.000277,"1.0|3.0"],
			"thickness"		=>["float",1.0,"0.1|3.0"],
			"sigmaA"		=>["srgb","000000","ColorPicker"],
			"specularTransmittance"=>["srgb","FFFFFF","ColorPicker"]
		}
		dfotherprops["other"] = {
			"bump"=>["srgb","","ColorPicker"],
			"twosided"=>["string","Yes","Yes|No"]
		}
		#"bump_scale"=>["float","1","0|3|0.01"],
		#"bump_width"=>["integer","1000","100|2000|100"],
		#"bump_height"=>["integer","1000","100|2000|100"],
		dfcoatprops["none"] = {}

		return dfprops , dfcoatprops ,	dflightprops , dfdipoleprops , dfotherprops
	end
	def export_mtl_material()
		#  @export_dir　共用エクスポート用ディレクトリ
		#  @scenefile:scene file 		共用シーンファイル名
		#  @objfile  :object file 		共用オブジェクトファイル名
		#  @mtlfile  :material file 	共用マテリアルライブラリファイル名
		#  PRECISION ：precision		共用数値フォーマット（小数点桁）
		#  SCALE_FACTOR:scale factor	共用スケール（インチをメートル）
		@matcodelist = Hash.new
		@textureslist = Hash.new
		#defdif = @defcolor.split(" ").collect{|dd| (dd.to_f)** 2.2 }.join(" ")
		#defdif = "0.65 0.65 0.65"# if @clay == "yes"
		dfprops , dfcoatprops ,	dflightprops , dfdipoleprops , dfotherprops = mitsuba_mat_properties_list()
		tempmatcode , tempmatcode2 , trans= export_this_material( nil , "t2h_su2objmtl_default_material" , dfprops , dfcoatprops , dflightprops , dfdipoleprops , dfotherprops )
		@matcodelist["t2h_su2objmtl_default_material"] = [ tempmatcode ,tempmatcode2 , trans]
#@matcodelist["t2h_su2objmtl_default_material"] = [ %Q{<bsdf type="twosided" id="MATERIAL_t2h_su2objmtl_default_material"><bsdf type="diffuse"><srgb name="reflectance" value="##{@defcolor}"/></bsdf></bsdf>\n} , "" ]
		@s2v_used_material.uniq!
		@s2v_volumes = []

		@s2v_used_material.each{|mat|
			next if mat == nil
			matname = enc_kconv( mat.name )
			if not @matcodelist[matname]
				tempmatcode , tempmatcode2 , trans = export_this_material( mat , matname , dfprops , dfcoatprops , dflightprops , dfdipoleprops , dfotherprops )
				@matcodelist[matname] = [ tempmatcode ,tempmatcode2 ,trans]
			end
		}
		txcode_buff = ""
		@textureslist.values.each{|txcode|
			txcode_buff << txcode + "\n"
		}
		@textureslist = nil
		return txcode_buff
	end
	def decim_to_length(lstr)##get_length_local
		#ダイアログの値をスケッチアップの長さに変換
		#if "#{1.to_f.to_l}".index(",") != nil
		#	lstr2 = lstr.gsub(/\./,",")
		#else
		#	lstr2 = lstr.gsub(/\,/,".")
		#end
		return 1.0 if not lstr
		lstr2 = lstr.gsub(/\,/,".")
		if 0.to_l.to_s["\""]
			ans = lstr2.to_f.inch
		elsif 0.to_l.to_s["'"]
			ans = lstr2.to_f.feet
		elsif 0.to_l.to_s["mm"]
			ans = lstr2.to_f.mm
		elsif 0.to_l.to_s["cm"]
			ans = lstr2.to_f.cm
		elsif 0.to_l.to_s["m"]
			ans = lstr2.to_f.m
		else
			ans = lstr2.to_f.inch
		end
		return ans
	end
	def length_to_decim(ln)
		#スケッチアップの長さをダイアログに表示するときに使う
		if 0.to_l.to_s["\""]
			ans = ln.to_f
		elsif 0.to_l.to_s["'"]
			ans = ln.to_f.to_feet
		elsif 0.to_l.to_s["mm"]
			ans = ln.to_f.to_mm
		elsif 0.to_l.to_s["cm"]
			ans = ln.to_f.to_cm
		elsif 0.to_l.to_s["m"]
			ans = ln.to_f.to_m
		else
			ans = ln.to_f
		end
		return ans
	end
	def export_this_material( mat , matname , dfprops , dfcoatprops , dflightprops , dfdipoleprops , dfotherprops, texscale = 1.0 )
		clothdir = File.join(File.dirname(__FILE__),"cloth")
		trans = false
		if mat and mat.texture != nil
			texturename = enc_kconv(File.basename(mat.texture.filename.to_s))
			if texturename[-4,4].downcase == ".png" or texturename[-4,4].downcase == ".jpg" 
				uscale = 1.0 / mat.texture.width / texscale
				vscale = 1.0 / mat.texture.height / texscale
				#if texturename[-4,4].downcase != ".mip"
					if @texnames_bymat[mat.name]
						texturename,useaplha = @texnames_bymat[mat.name]
					end
					texturename_withdir = texturename
					if @texture_exportpath != ""
						texturename_withdir = File.join( @texture_exportpath , texturename  )
					end
				#else
				#	texturename = ""
				#end
			else
				texturename = ""
			end
		end

		if texturename and texturename != "" and not @textureslist[texturename]
			@textureslist[texturename] = %Q{<texture type="bitmap" id="#{texturename}">
			<string name="filename" value="#{texturename_withdir}"/>
			<float name="uscale" value="#{uscale}" /><float name="vscale" value="#{vscale}" />
			<float name="uoffset" value="0.0" /><float name="voffset" value="#{-vscale}" />
			<boolean name="cache" value="false" />
			</texture>\n}
		end
		props = nil
		props ,coatprops,lightprops,dipoleprops = get_prop_from_material(mat)
		
		dfsrgb = props["default_mat_color"]||"FFFFFF" #default material color
		props["twosided"] = @twosided  #141123
		if not props or not props["bsdf_type"]
			props = {}
			#props["twosided"] = "Yes"
			props["bsdf_type"] = "diffuse"
			#props["mask"] = "No"
		end
		if not props["bsdf_type"].include?("dielectric")
			if mat and mat.alpha < 1.0
				#ガラス系でないBSDFマテリアルで透明度がある場合、mixtureを使って半透明にする
				props["mixturebsdf"] = "Yes"
				props["twosided"] = "Yes"	#<=多分こうした方が良さそう
				if mat.alpha < 0.2#透明度が20%以下の場合は非表示対応
					trans = true
				end
			end
		end
		if props["bsdf_type"] == "thindielectric"
			trans = true#thindielectricは非表示対応
		end
		if not coatprops
			coatprops["coat_type"] = "none"
		end

		if not lightprops
			lightprops["light_type"] = "none"
		end
		if not dipoleprops
			dipoleprops["dipole_type"] = "none"
		end

		#get_mat_property_att( props )
		bsdf = props["bsdf_type"]
		
		bsdf = "diffuse" if not dfprops[bsdf]
		bsdfname = bsdf.split("-")[0]
		bsdfname2 = bsdf.split("-")[1]
		#props["twosided"] = true
		tempmatcode = ""
		endcode = ""
		name_on = false
		matid = "MATERIAL_#{matname}"
		if props["mixturebsdf"] == "Yes"
			lightprops["light_type"] = "none"
			#emitterを使っている場合は透明を使えない
			tempmatcode << %Q{<bsdf type="mixturebsdf" }
			if name_on == false
				name_on = true
				tempmatcode << %Q{id="#{matid}">\n}
			else
				tempmatcode << %Q{>\n}
			end
			if mat
				malpha = mat.alpha
			else
				malpha = 0
			end
			tempmatcode << %Q{	<string name="weights" value="#{1.0-malpha}, #{malpha}"/>\n}
			tempmatcode << %Q{	<bsdf type="thindielectric">\n}
			tempmatcode << %Q{		<float name="intIOR" value="1.0"/>\n}
			tempmatcode << %Q{		<float name="extIOR" value="1.0"/>\n}
			tempmatcode << %Q{	</bsdf>\n}

			endcode = '</bsdf>' + endcode
		end
		if useaplha == true ##props["mask"] == "Yes"
			if texturename_withdir and texturename_withdir != ""
				tempmatcode << %Q{<bsdf type="mask" }
				if name_on == false
					name_on = true
					tempmatcode << %Q{id="#{matid}">\n}
				else
					tempmatcode << %Q{>\n}
				end
				tempmatcode << %Q{<texture name="opacity" type="bitmap">\n}
				tempmatcode << %Q{<string name="filename" value="#{texturename_withdir}"/>
				<float name="uscale" value="#{uscale}" /><float name="vscale" value="#{vscale}" />
				<float name="uoffset" value="0.0" /><float name="voffset" value="#{-vscale}" />
				<string name="channel" value="a"/>
				</texture>\n}
				endcode = '</bsdf>' + endcode
			end
		end
		#bwidth = props["bump_width"]||"1000mm"
		#bheight = props["bump_height"]||"1000mm"
		
		#buscale = 1 / bwidth.to_f.mm.to_inch / texscale
		#bvscale = 1 / bheight.to_f.mm.to_inch / texscale
		
		#buscale = 1 / decim_to_length(bwidth) / texscale
		#bvscale = 1 / decim_to_length(bheight) / texscale
		
		###if texturename_withdir and texturename_withdir != ""
		if props["bump"] and props["bump"] != ""
			bumpfile = props["bump"]||""
			txfile_arr = bumpfile.to_s.split("?")
			bumpfile = txfile_arr[0]
			if bumpfile == "*"
				if texturename_withdir and texturename_withdir != ""
					bumpfile = texturename_withdir
					buscale = uscale
					bvscale = vscale
					if txfile_arr.size > 2
						bumpscale = (decim_to_length(txfile_arr[3])||10.0).to_f
					end
				else
					bumpfile = ""
				end
			elsif /^[A-F0-9][A-F0-9][A-F0-9][A-F0-9][A-F0-9][A-F0-9]$/i =~ bumpfile.to_s
				bumpfile = ""
			elsif bumpfile.to_s == ""
				bumpfile = ""
			else
				bumpfile=txfile_arr[0]
				if File.exists?(bumpfile)
					buscale = uscale.to_f
					bvscale = vscale.to_f
					if txfile_arr.size > 0
						buscale = ((1.0 / decim_to_length(txfile_arr[1]) / texscale)||uscale).to_f
						buscale = 0.01 if buscale <= 0.0
					end
					if txfile_arr.size > 1
						bvscale = ((1.0 / decim_to_length(txfile_arr[2]) / texscale)||vscale).to_f
						bvscale = 0.01 if bvscale <= 0.0
					end
					if txfile_arr.size > 2
						bumpscale = (decim_to_length(txfile_arr[3])||10.0).to_f
					end
				else
					bumpfile = ""
				end
			end

			if bumpfile != ""
				#bumpscale = props["bump_scale"]||1
				bmscale = (bumpscale || 10.0).to_f/1000.to_f
				if bmscale > 0.0
					tempmatcode << %Q{<bsdf type="bumpmap" }
					if name_on == false
						name_on = true
						tempmatcode << %Q{id="#{matid}">\n}
					else
						tempmatcode << %Q{>\n}
					end
					tempmatcode << %Q{<texture type="scale"><float name="scale" value="#{bmscale}"/><texture type="bitmap">\n}
					tempmatcode << %Q{<string name="filename" value="#{bumpfile}"/>}
					tempmatcode << %Q{<float name="uscale" value="#{buscale}" /><float name="vscale" value="#{bvscale}" />\n}
					tempmatcode << %Q{<float name="uoffset" value="0.0" /><float name="voffset" value="#{-bvscale}" /></texture></texture>\n}
					endcode = '</bsdf>' + endcode
				end
			end
		end
		###end
		if coatprops["coat_type"] and coatprops["coat_type"] != "none"
			coatype = coatprops["coat_type"]
			tempmatcode << %Q{<bsdf type="#{coatype.to_s}" }
			if name_on == false
				name_on = true
				tempmatcode << %Q{id="#{matid}"}
			end
			tempmatcode << %Q{>\n}
			dfcoatprops[coatype].keys.each{|dfkey|
				prop = coatprops[dfkey]||dfcoatprops[coatype][dfkey][1]
				dftype = dfcoatprops[coatype][dfkey][0]
				dflist = dfcoatprops[coatype][dfkey][2]
				tempmatcode << matcoder( mat , dftype , dfkey , prop , dflist , false , texturename ,matname,buscale,bvscale,dfsrgb )
			}
			#tempmatcode << %Q{<bsdf type="#{bsdf}">\n}
			endcode = "</bsdf>" + endcode
		end

		if ["dielectric","thindielectric","roughdielectric","difftrans","hk"].index(props["bsdf_type"])
			lightprops["light_type"] = "none"
			#emitterを使っている場合は透明を使えない
			tempmatcode << %Q{<bsdf type="#{bsdfname}" }
			if name_on == false
				tempmatcode << %Q{id="#{matid}"}
			end
			tempmatcode << %Q{>\n}
			endcode = "</bsdf>"  + endcode
		elsif props["twosided"] == "Yes"
			tempmatcode << %Q{<bsdf type="twosided" }
			if name_on == false
				tempmatcode << %Q{id="#{matid}"}
			end
			tempmatcode << %Q{>\n}
			tempmatcode << %Q{<bsdf type="#{bsdfname}">\n}
			endcode = "</bsdf></bsdf>" + endcode
		else
			tempmatcode << %Q{<bsdf type="#{bsdfname}" }
			if name_on == false
				tempmatcode << %Q{id="#{matid}"}
			end
			tempmatcode << %Q{>\n}
			endcode = "</bsdf>" + endcode
		end
		if bsdfname == "irawan"
			tempmatcode << %Q{<string name="filename" value="#{File.join(clothdir,bsdfname2)}.wv"/>\n}
		end
		dfprops[bsdf].keys.each{|dfkey|
			if dipoleprops["dipole_type"] == "dipole" and ( dfkey == "diffuseReflectance" or dfkey == "reflectance" )
				prop = "000000"
				dftype = "srgb"
				dflist = "ColorPicker"
			else
				prop = props[dfkey]||dfprops[bsdf][dfkey][1]
				dftype = dfprops[bsdf][dfkey][0]
				dflist = dfprops[bsdf][dfkey][2]
			end
			tempmatcode = tempmatcode + matcoder( mat , dftype , dfkey , prop , dflist , true , texturename ,matname,buscale,bvscale,dfsrgb )
		}

		tempmatcode = tempmatcode + endcode + "\n"
		tempmatcode2 = ""
		if lightprops["light_type"] == "arealight_blackbody"
			templ = lightprops["temperature"]||6000
			sw = lightprops["samplingWeight"]||1
			sc = lightprops["scale"]||-3.0
			if sc.to_f < 0
				scstr = "1e" + sc.to_s
			else
				scstr = "1e+" + sc.to_s
			end
			tempmatcode2 << '<emitter type="area"><blackbody name="radiance" temperature="' + templ + 'K" scale="' + scstr + '" />
			<float name="samplingWeight" value="' + sw + '" /></emitter>'
		elsif lightprops["light_type"] == "arealight_srgb"
			colorcode = lightprops["radiance"]||"FFFFFF"
			sw = lightprops["samplingWeight"]||1
			sc = lightprops["scale2"]||1.0
			rgb = srgb_to_rgb( colorcode )
			rgb2 = rgb.map{|i| i = i * sc.to_f ** 2 }
			tempmatcode2 << '<emitter type="area"><rgb name="radiance" value="' + rgb2.join(",") + '"/>
			<float name="samplingWeight" value="' + sw.to_s + '" /></emitter>'
		end
		if dipoleprops["dipole_type"] == "dipole"
			dipoletype = dipoleprops["dipole_type"]
			tempmatcode2 << '<subsurface type="' + dipoletype + '">' + "\n"
			dfdipoleprops[ dipoletype ].keys.each{|dfkey|
				prop = dipoleprops[dfkey]||dfdipoleprops[dipoletype][dfkey][1]
				dftype = dfdipoleprops[dipoletype][dfkey][0]
				dflist = dfdipoleprops[dipoletype][dfkey][2]
				tempmatcode2 << matcoder( mat , dftype , dfkey , prop , dflist , false , texturename,matname,buscale,bvscale,dfsrgb )
			}
			tempmatcode2 << '</subsurface>' + "\n"

		end
		return tempmatcode , tempmatcode2 , trans
	end
	def matcoder( mat , dftype , dfkey , prop , dflist , chk = true , texturename = "" , matname = "" , buscale = 100 , bvscale = 100 , dfsrgb = "FFFFFF")
		texscale = 1.0
		matcode = ""
		if not mat
			matcolor = srgb_to_color(dfsrgb)
		else
			matcolor = mat.color
		end
		if dflist == "Origin"
			if texturename and texturename != ""
				matcode = '<ref name="' + dfkey + '" id="' + texturename + '"/>' + "\n"
			else
				if mat and dfkey == "specularTransmittance" and chk == true
					color = Sketchup::Color.new( 255,255,255 ).blend( matcolor , 1.0 - mat.alpha )
				else
					color = matcolor
				end
				colorcode = color.to_a[0..2].collect{|x| "%02X" % x}.join
				if dftype == "srgb"
					matcode = '<srgb name="' + dfkey + '" value="#' + colorcode + '"/>' + "\n"
				else
					rgb = srgb_to_rgb( colorcode )
					matcode = '<rgb name="' + dfkey + '" value="' + rgb.join(",") + '"/>' + "\n"
				end
			end
		elsif dflist == "ColorPicker"
			txfile_arr = prop.to_s.split("?")
			mapfile = txfile_arr[0]
			if mapfile == "*"
				if texturename and texturename != ""
					matcode = '<ref name="' + dfkey + '" id="' + texturename + '"/>' + "\n"
				else
					color = matcolor
					colorcode = color.to_a[0..2].collect{|x| "%02X" % x}.join
					if dftype == "srgb"
						matcode = '<srgb name="' + dfkey + '" value="#' + colorcode + '"/>' + "\n"
					else
						rgb = srgb_to_rgb( colorcode )
						matcode = '<rgb name="' + dfkey + '" value="' + rgb.join(",") + '"/>' + "\n"
					end
				end
			elsif /^[A-F0-9][A-F0-9][A-F0-9][A-F0-9][A-F0-9][A-F0-9]$/i =~ mapfile
				if dftype == "srgb"
					matcode = '<srgb name="' + dfkey + '" value="#' + prop.to_s + '"/>' + "\n"
				else
					rgb = srgb_to_rgb( prop.to_s )
					matcode = '<rgb name="' + dfkey + '" value="' + rgb.join(",") + '"/>' + "\n"
				end
			elsif mapfile == ""
				matcode = '<srgb name="' + dfkey + '" value="#FFFFFF"/>' + "\n"
			else
				mapname = matname + "_" + dfkey + "_texture"
				if not @textureslist[mapname]
					txuscale = buscale.to_f
					txvscale = bvscale.to_f
					if txfile_arr.size > 0
						txuscale = ((1.0 / decim_to_length(txfile_arr[1]) / texscale).to_f||txuscale)
						txuscale = 0.01 if txuscale <= 0.0
					end
					if txfile_arr.size > 1
						txvscale = ((1.0 / decim_to_length(txfile_arr[2]) / texscale).to_f||txvscale)
						txvscale = 0.01 if txvscale <= 0.0
					end
					@textureslist[mapname] = %Q{<texture type="bitmap" id="#{mapname}">
					<string name="filename" value="#{mapfile}"/>
					<float name="uscale" value="#{txuscale}" /><float name="vscale" value="#{txvscale}" />
					<float name="uoffset" value="0.0" /><float name="voffset" value="#{-txvscale}" />
					</texture>\n}
				end
				matcode = '<ref name="' + dfkey + '" id="' + mapname + '"/>' + "\n"
			end
		else
			matcode = '<' + dftype.to_s + ' name="' + dfkey + '" value="' + prop.to_s + '"/>' + "\n"
		end
		return matcode
	end
	def srgb_to_rgb( srgb )
		srgb2 = srgb.rjust(6,"0")
		rgb = []
		rgb[0] = ( srgb2[0,2].hex.to_f / 255.0 ) ** 2.2
		rgb[1] = ( srgb2[2,2].hex.to_f / 255.0 ) ** 2.2
		rgb[2] = ( srgb2[4,2].hex.to_f / 255.0 ) ** 2.2
		return rgb
	end
	def srgb_to_color( srgb )
		srgb2 = srgb.rjust(6,"0")
		color = Sketchup::Color.new( srgb2[0,2].hex,srgb2[2,2].hex,srgb2[4,2].hex )
		return color
	end

end
end
end
